"""Hensel encoder/decoder module."""

import ast

from tropic.common.const import oo


def encode(inp, base):
    """Hensel decoding function.

    Args:
        inp (list): list of p-adic coefficients
        base (int): base of p-adic encoding

    Returns:
        int: hensel code
    """
    rinp = enumerate(reversed(inp))
    monomials = [coef * base ** power for power, coef in rinp]
    return sum(monomials)


def decode(inp, base, prec=-1):
    """Hensel decoding function.

    Args:
        inp (int): scalar to be represented as hensel code
        base (int): base of encoding
        prec (int): positive integer, the p-adic precision of the root

    Returns:
        list: list of coefficients of hensel code
    """
    out = []
    if inp in {0, 1}:
        out += [inp]
    elif abs(inp) == oo:
        out += [inp]
    else:
        while inp:
            out += [int(abs(inp) % base)]
            inp //= base
        out.reverse()
    pads = [0 for _ in range(prec - len(out))]
    return pads + out


def num2str(inp):
    """Hensel decode function.

    Args:
        inp (int): scalar to be represented as hensel code

    Returns:
        str: list of coefficients of hensel code
    """
    sgn = '-' if inp < 0 else '+'
    pinp = abs(inp)
    return ''.join([sgn, 'oo']) if pinp == oo else str(inp)


def str2num(inp):
    """Hensel decode function.

    Args:
        inp (int): scalar to be represented as hensel code

    Returns:
        float: list of coefficients of hensel code
    """
    sgn = '-' if inp < 0 else ''
    inf = ''.join([sgn, 'oo'])
    return inf if abs(inp) == oo else str(inp)


def lst2str(inp, sep='.'):
    """Hensel decode function.

    Args:
        inp (int): scalar to be represented as hensel code
        sep (str): base of encoding

    Returns:
        str: list of coefficients of hensel code
    """
    return sep.join(map(str, inp))


def str2lst(inp):
    """Hensel decode function.

    Args:
        inp (str): scalar to be represented as hensel code

    Returns:
        str: list of coefficients of hensel code
    """
    return map(float, ast.literal_eval(inp))



"""Multi-level matrix core."""

import numpy
import tensorflow as tf
from tabulate import tabulate


def init(shape):
    """Do counting of number of levels of mutli-level matrix.

    Args:
        shape (tuple): shape of a new tensor

    Returns:
        tensor: new tensor filled with zeros
    """
    return tf.zeros(shape, dtype=tf.float32)


def pretty(inp):
    """Do pretty printing of mutli-level matrix.

    Args:
        inp (tensor): mutli-level matrix

    Returns:
        str: pretty string representation
    """
    if levels(inp) > 0:
        rows, cols = inp.shape[:2]
        table = numpy.ndarray((rows, cols), dtype=object)
        for row in range(rows):
            for col in range(cols):
                table[row, col] = pretty(inp[row, col])
        return tabulate(table, tablefmt=_format(inp))
    return _signum(inp) + str(inp.numpy())


def levels(matrix):
    """Do counting of number of levels of mutli-level matrix.

    Args:
        matrix (tensor): odd-dimensional tensor

    Returns:
        int: number of levels
    """
    return int(len(matrix.shape) / 2)


def random(shape):
    """Create random mutli-level matrix.

    Args:
        shape (tuple): shape of a matrix

    Returns:
        tensor: random matrix
    """
    matrix = tf.random.uniform(shape, minval=0, maxval=10, dtype=tf.int32)
    return tf.cast(matrix, dtype=tf.float32)


def _format(inp):
    return 'plain' if levels(inp) % 2 else 'grid'


def _signum(inp):
    return '-' if numpy.sign(inp.numpy()) == -1 else ''



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tropic', 'tropic.codecs', 'tropic.common', 'tropic.linalg']

package_data = \
{'': ['*']}

install_requires = \
['logzero>=1.5.0,<2.0.0', 'tabulate>=0.8.7,<0.9.0', 'tensorflow>=2.3.0,<3.0.0']

setup_kwargs = {
    'name': 'tropic',
    'version': '0.2.2',
    'description': '',
    'long_description': None,
    'author': 'skip',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
